#ifndef FrameCPP_VERSION_8_FrSimEvent_HH
#define FrameCPP_VERSION_8_FrSimEvent_HH

#include <string>
#include <vector>

#include "framecpp/FrameCPP.hh"

#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version7/FrSimEvent.hh"

#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/GPSTime.hh"
#include "framecpp/Version8/FrVect.hh"
#include "framecpp/Version8/FrTable.hh"
#include "framecpp/Version8/STRING.hh"

namespace FrameCPP
{
  namespace Version_8
  {
    //===================================================================
    // FrSimEventNPS
    //===================================================================
    typedef Previous::FrSimEvent::FrSimEventNPS FrSimEventNPS;

    //===================================================================
    // FrSimEventPST
    //===================================================================

    typedef Version_4::FrSimEventPST< PTR_STRUCT,
				      FrVect, FrTable >
    FrSimEventPS;

    //===================================================================
    // FrSimEvent
    //===================================================================
    class FrSimEvent
      : public ObjectWithChecksum< Common::FrameSpec::Object >,
	public FrSimEventNPS,
	public FrSimEventPS,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrSimEvent( );
      FrSimEvent( const FrSimEvent& eventData );
      FrSimEvent( const std::string& name, const std::string& comment,
		  const std::string& inputs,
		  const GPSTime& time,
		  const REAL_4 timeBefore,
		  const REAL_4 timeAfter,
		  const REAL_4 amplitude,
		  const ParamList_type& parameters );
      FrSimEvent( const Previous::FrSimEvent& Source,
		  Common::IStream* Stream );

      const std::string& GetNameSlow( ) const;

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      virtual const char* const ObjectStructName( ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

      //: Merge with another FrSimEvent
      FrSimEvent& Merge( const FrSimEvent& RHS );

      /// \brief comparison operator
      bool operator==( const FrSimEvent& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      //-----------------------------------------------------------------
      cmn_streamsize_type pBytes( const Common::StreamBase& Stream) const;

      virtual FrSimEvent* pCreate( Common::IStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      //-----------------------------------------------------------------
      virtual void pWrite( Common::OStream& Stream ) const;

    private:
      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrameSpec::Object > object_type;

      using Common::FrameSpec::Object::Create;

      FrSimEvent( Common::IStream& Stream );
    };

    inline std::auto_ptr< Common::FrameSpec::Object > FrSimEvent::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrSimEvent, FrSimEvent >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrSimEvent::
    StructName( )
    {
      static const char* const name = "FrSimEvent";
      return name;
    }

    inline bool FrSimEvent::
    operator==( const FrSimEvent& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && (m_refs == RHS.m_refs )
		    ) );
    }
  } // namespace - Version_8
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_8_FrSimEvent_HH */
